---
title: "NuGet (.NET) Packages and Feeds in Proget"
nav-title: "NuGet (.NET)"
order: 3
---

NuGet is a package format developed by Microsoft to distribute free and open source. NET libraries. Usually, these packages are publicly available on NuGet.org and are used by Visual Studio or the nuget.exe command-line client.

- [NuGet command-line Reference](https://docs.microsoft.com/en-us/nuget/tools/nuget-exe-cli-reference)

By creating a [feed](/docs/proget/feeds/feed-overview) in ProGet, you can use NuGet packages to distribute private proprietary libraries within your organization. Then configure [Visual Studio](/docs/proget/feeds/nuget/howto-nuget-proxy#add-visual-studio) or the [NuGet CLI](/docs/proget/feeds/nuget/howto-nuget-proxy#add-cli) to use your feed as a new package source.

You can also [use ProGet as a proxy for NuGet.org](/docs/proget/feeds/nuget/howto-nuget-proxy), both to block [improper licenses](/docs/proget/sca/licenses) and [security vulnerabilities](/docs/proget/sca/vulnerabilities), and to cache packages in case the Internet or NuGet.org goes down.

## Installing Packages

NuGet packages can be installed using either the [NuGet CLI](https://docs.microsoft.com/en-us/nuget/quickstart/install-and-use-a-package-using-the-dotnet-cli) or Visual Studio. To install a package from a ProGet feed in the NuGet CLI, `use the dotnet nuget install` command:

```bash
$ dotnet nuget install «package-name» -Version «package-version» -Source http://«proget-server»/nuget/«feed-name»/
```

For Visual Studio you will first need to make sure your ProGet Feed is [set as a source](/docs/proget/feeds/nuget/howto-nuget-proxy#add-visual-studio). Then Right-click on your project in the Solution Explorer and select Manage NuGet Packages.

In the NuGet Package Manager window go to the "Browse" tab and search for the package you want to install. Then select the package from the list and click "Install".

After installing, it will be added to your project’s dependencies, and you can start using it in your code.

## Authenticating to NuGet Feeds

NuGet client tools (such as nuget.exe and NuGet Package Explorer) have the option to use a "NuGet API Key" as an authentication mechanism for publishing packages. This makes a lot of sense for the NuGet Gallery, since it is a community-run site where anyone can publish packages that they own. However, in enterprise environments, the concept of "individual package ownership" cannot be fully implemented.

In ProGet, a principal (user or group) either has privileges to publish a package to a feed or not. This allows ProGet administrators to more easily control access and usage through LDAP and groups, rather than through community mechanisms like API Keys. However, if you want to use a NuGet API key when pushing to ProGet, you have two options:

### Using an API Key

You can [Create ProGet API Key](/docs/proget/reference-api/proget-apikeys) with Feed Access; you can further restrict this key by associating it to a user you've already given specific permissions

When creating an API Key, fill in the fields by selecting "Feeds ("Use Certain Feeds)" as the "Feed Type" and selecting the NuGet feed to authenticate to. Make sure that the required permission boxes are checked, and then select "Save".

![](/resources/docs/proget-nuget-api.png){height="" width="50%"}

To add and authenticate the feed in The NuGet CLI, enter the following:

```bash
$ dotnet nuget setApiKey «api-key» -Source «feed-url»
```

### Using username:password as an API Key

ProGet will allow users to authenticate in this mode using an API Key in the format: `«username:password»`:

```bash
$ dotnet nuget push «package-name» -Source «feed-url» -ApiKey «username:password»
```

For example, pushing the package `MyPackage.nupkg` to `http://proget.corp.local/nuget/internal-nuget/v3/index.json`, with the username `john` and password `abc12345`, enter: 

```bash
$ dotnet nuget push MyPackage -Source http://proget.corp.local/nuget/internal-nuget/v3/index.json -ApiKey john:abc12345
```

### Authenticating in Visual Studio

Teams can require authentication for feeds in ProGet, which means that users must log in to their package source in ProGet. To do this, You will first need to create a "Personal API Key". Under the user icon in the top-right of ProGet, select “Personal API Keys” and then click "Create Personal API Key".

![proget-personalapikey-create.png](/resources/docs/proget-personalapikey-create.png)

In the pop-up, type an appropriate display name and click "Create API Key" Then, click on the display name and the pop-up will appear again, this time with the Personal API key generated.

![proget-createpersonalapikey.png](/resources/docs/proget-createpersonalapikey.png)

When checking the connection of a private NuGet feed after [adding it as a source](/docs/proget/feeds/nuget/howto-nuget-proxy#add-visual-studio), a pop-up window will appear asking you to connect to the server. Enter "api" as the username and the personal API key as the password. 

![visualstudio-connecttoserver-inputted.png](/resources/docs/visualstudio-connecttoserver-inputted.png)

Finally, click "OK". This will configure the Authenticated NuGet feed in Visual Studio. 

## Creating and Publishing NuGet Packages 

The [NuGet package format](https://docs.microsoft.com/en-us/nuget/schema/nuspec) is well-documented, and you can create packages in a number of ways, including [directly from Visual Studio](https://docs.microsoft.com/en-us/nuget/guides/create-net-standard-packages-vs2017) or using [nuget.exe](https://docs.microsoft.com/en-us/nuget/create-packages/creating-a-package). 

You can read more about creating and publishing packages in our [HOWTO: Publish NuGet Packages to a Private Repository in ProGet](/docs/proget/feeds/nuget/howto-nuget-publish) guide.

:::(Info) (CI/CD for NuGet Packages!)
Check out the [NuGet Packages documentation in BuildMaster](/docs/buildmaster/development-platforms/buildmaster-platforms-dotnet/buildmaster-platforms-dot-net-nuget) to learn how to apply the principles of Continuous Integration & Continuous Delivery (CI/CD) to quickly test and deploy packages you create along with your application
:::

## Deleting and Unlisting NuGet Packages 

You can delete (permanently remove) or unlist (hide from most search results) NuGet packages from your feed by navigating to the package page and clicking the appropriate Delete Package or Unlist Package from List button. These actions require the `Feeds_DeletePackage` or `Feeds_UnlistPackage`  permission attribute, respectively.

To programmatically delete a package from your feed, you can use the NuGet CLI's [delete command](https://docs.microsoft.com/en-us/nuget/reference/cli-reference/cli-ref-delete), or make a `DELETE` request via HTTP:

```bash
$ DELETE http://{proget-server}/nuget/{feed-name}/package/{ID}/{VERSION}`
```

Note that this behavior is different than [NuGet.org's DELETE command](https://docs.microsoft.com/en-us/nuget/api/package-publish-resource#delete-a-package), which unlists packages instead.

To programmatically unlist (or relist) a package, you can use the `NuGetPackagesV2_SetListed` method within the [Native API](/docs/proget/reference-api/proget-api-http).

## Repackaging NuGet Packages

[Repackaging](/docs/proget/packages/repackaging) is a process of changing a package's version without changing its contents, enabling use cases such as converting a pre-release package to a stable version, and more.

ProGet supports NuGet repackaging natively. To view an example of how it can be accomplished using the [Repackaging API](/docs/proget/packages/repackaging) in a CI/CD system, you can view an [example application](https://buildmaster.inedo.com/applications/2/overview) in our public BuildMaster instance.

## Using the NuGet API

ProGet implements the  [NuGet Server API](https://learn.microsoft.com/en-us/nuget/api/overview), which is extensively documented by Microsoft.

:::(Info) (⚙ NuGet API Endpoint URL)
 `«proget-base»/nuget/«feed-name»/v3/index.json`
:::

This is referred to as the JSON-LD (v3) API. ProGet also implements the older NuGet API, called the [ODATA (v2) API](/docs/proget/feeds/nuget/legacy-nuget#nuget-odata-v2-api).

## CI Packages and NuGet Caching
The NuGet client maintains a 30-minute HTTP cache for package restore responses. In situations where you deploy more than one new version of a package within 30 minutes, and produce a CI build referencing each of those packages within that 30-minute window, your build will likely fail with a package not found error. This is a known issue in NuGet, and can be worked around by deleting the http cache on your build server prior to restoring packages:

```bash
$ dotnet nuget locals http-cache --clear
```

See [NuGet GitHub Issue 3116](https://github.com/NuGet/Home/issues/3116) for more information about this error.

## Symbols and Source Packages 

A NuGet feed in ProGet may be configured as a Symbol/Source server compatible with debuggers such as Visual Studio and WinDbg. See [Symbols and Source Code in ProGet](/docs/proget/feeds/nuget/symbol-and-source-server) for more information.

## Vulnerability Integration With Visual Studio and The NuGet CLI

Beginning in ProGet 2023.5, we have added support to show [vulnerability information](https://devblogs.microsoft.com/nuget/how-to-scan-nuget-packages-for-security-vulnerabilities/) directly in Visual Studio and the NuGet CLI.  You will see the severity of your vulnerability along with a link to see the vulnerability details in ProGet.

When selecting a package version with a vulnerability in Visual Studio, you will see the severity with a clickable link to the vulnerability in ProGet.

![](/resources/docs/proget-nuget-vulnerability-visual-studio.png){height="" width=""}

When running `dotnet list package --vulnerable` in the NuGet CLI, you will see a list of vulnerable packages with their severity and a link to the vulnerability in ProGet.

![](/resources/docs/proget-nuget-vulnerability-nuget-cli.png){height="" width=""}

:::(INFO) (Note:)
ProGet Free edition limits vulnerability information shown in Visual Studio and the NuGet CLI.
:::

## Deprecation Integration With Visual Studio and The NuGet CLI

ProGet will show deprecation information directly in Visual Studio and the NuGet CLI.  You will see if a specific version of the package was deprecated along with the specified reason. This works for both packages that come from a NuGet connector and packages that were deprecated using the ProGet UI.

When selecting a package version that has been deprecated in Visual Studio, you will see a warning that the package has been deprecated. If the package has been installed in your project, you will also see a shield in the package list.

![](/resources/docs/nuget-deprecated-visualstudio.png){height="" width=""}

When running `dotnet list package --deprecated` in the NuGet CLI, you will see a list of deprecated packages installed in your project along with the deprecation reason.

![](/resources/docs/nuget-deprecated-cli.png){height="" width=""}

## Troubleshooting

### NuGet Connectors

The following applies to NuGet.org [deprecating API features](https://devblogs.microsoft.com/nuget/custom-v2-odata-queries-will-be-deprecated-march-9-2021/) on **March 9, 2021**, and may help explain problems with connectors as a result.

#### My NuGet feed is suddenly empty and not showing remote packages

If you configured a connector to NuGet.org in ProGet 5.2 or earlier, it may suddenly stop working and not return any packages. This is due to an API change in NuGet.org, and the only solution is to upgrade to ProGet 5.3 or later. You can also work around the problem temporarily by [disabling your connector filters](/docs/proget/feeds/connector-overview#connector-filters).

#### My NuGet connector shows healthy, but it has the wrong package count

If you configured a connector to NuGet.org in ProGet 5.2 or earlier, you may notice a healthy connector status but an incorrect package count. NuGet.org has deprecated its previous queries to check the status and count of NuGet packages because they are very taxing on the NuGet servers. In the future, NuGet.org will display a fixed number of packages when health status is queried.

### NuGet API Background

NuGet is a package format developed by Microsoft to distribute free and open-source .NET libraries. Usually, these packages are publicly available on NuGet.org and are used by Visual Studio or the command line client nuget.exe.

When NuGet was released, a client/server protocol was needed for users to query packages, retrieve package metadata, and download package artifacts. The logical choice at the time was [OData](http://www.odata.org): by exposing their Entity Framework model via OData, the NuGet client could run any query on the database, using OData as the protocol. The problem with this architecture is that every query is a direct database query. This caused significant performance issues with NuGet.org, causing the server to crawl under heavy load.

In early 2016, NuGet introduced the third version of its API to dramatically improve the overall performance of NuGet.org.

In late 2019, NuGet announced that they would begin deprecating features of their API v2 to make it a stable and performant shim on API v3, ensuring that the data returned in both APIs is identical without breaking backward compatibility. The first step was to throttle the usage of API v2. The second step of this process is to eliminate non-performing queries that do not match API v3 on a 1:1 basis.

On March 9, 2021, NuGet.org began blocking select endpoints used by third-party clients such as ProGet. For more information, Microsoft's blog article [Custom V2 OData queries will be deprecated on March 9, 2021](https://devblogs.microsoft.com/nuget/custom-v2-odata-queries-will-be-deprecated-march-9-2021/).

#### How Does This Affect ProGet?

If you have already upgraded to ProGet 5.3+, you are not directly affected, but we recommend that you make sure that the v3 API is enabled in ProGet on the feed Manage Feed page, and configure your NuGet clients (and Visual Studio) to use the new v3 API URL for your feeds.

In ProGet 5.3, we introduced the implementation of the NuGet v3 API and gave connectors the ability to use the v3 API to connect to NuGet.org. ProGet 5.3 is also backward compatible with the NuGet v2 API. This means that any v2 client that connects to ProGet to query, push, pull, etc. will not be affected as long as those packages are local or their connector uses the v3 API. If you are still using a v2-based connector, ProGet 5.3 (via [PG-1847](https://issues.inedo.com/youtrack/issue/PG-1847)) will display a message if an unsupported query is used. Unsupported queries can be viewed in the Diagnostics Center under the "NuGet v2 Bad Requests" message category. This message can be turned off from the Manage Feed page of the respective feed.

If you are using ProGet 5.2 or lower, you will notice that some features of the ProGet connectors no longer work (see above). Nothing will change in your local packages, they will continue to work as before. The state of the connectors may be displayed correctly, but the total number of NuGet packages may not be the correct number of packages.

### Deprecated Packages in Visual Studio and NuGet CLI

Visual Studio and the NuGet CLI heavily caches this information and may take time to show this information after a package has been marked as deprecated. Due to this caching, parts of Visual Studio (version drop-down, version details, and package list) may show the deprecation information at different times. 

To force the NuGet CLI to update its deprecated packages, you will need to first clear the NuGet HTTP Cache using the command `dotnet nuget locals http-cache --clear`. Then the next time you run `dotnet list package --deprecated,` the deprecation information will show immediately.

To force Visual Studio to update its deprecated packages, you will need to navigate to "Tools" > "Options" > "NuGet Package Manager" > "General" and then click "Clear All NuGet Storage". This will clear the entire NuGet cache (including cached packages) from the machine and then the deprecated information will show immediately.