---
title: "HOWTO: Create and Upload Nuget Packages to a Private Repository in ProGet"
order: 2
---

ProGet let's you set up private NuGet repositories that will allow you to publish, store and share your internal NuGet packages within your organization.

This guide will walk you through the process of setting up a NuGet ["Feed"](/docs/proget/feeds/feed-overview) in ProGet as a private, custom repository. We'll also cover how to create, push, and install NuGet packages from this repository.

## Step 1: Create and Name a NuGet Feed

Start by creating a NuGet feed to host your internal packages. Navigate to "Feeds" and "Create New Feed". Then select "NuGet (.NET) Packages".

![](/resources/docs/proget-newfeed-nugetselect.png){height="" width="50%"}

Now select "No Connectors (Private packages only)" as this feed will be intended to host private packages.

![](/resources/docs/proget-nuget-noconnectors.png){height="" width="50%"}

From here, we name our feed. For this example, we will call it `internal-nuget`, and then click "Create Feed".

![](/resources/docs/proget-nuget-internal-name.png){height="" width="50%"}

You'll then see several options related to ProGet's [Vulnerability Scanning and Blocking](/docs/proget/sca/vulnerabilities) features. These are only for users looking to use third-party OSS packages. Leave these boxes unchecked, and select "Set Feed Features". You will then be directed to the new `internal-proget` feed, currently empty.

![](/resources/docs/proget-nuget-internal-empty.png){height="" width="50%"}

## Step 2: Create a NuGet Package

To create a NuGet package, create the project in `dotnet` CLI by entering:

```bash
$ dotnet new classlib -n MyPackage 
$ cd MyPackage 
```

In the project folder, create a `.nuspec` file, that will contain the package metadata. For example:

```xml
<?xml version="1.0"?> 
<package > 
  <metadata> 
    <id>MyPackage</id> 
    <version>1.0.0</version> 
    <authors>Your Name</authors> 
    <owners>Your Organization</owners> 
    <description>A simple example NuGet package.</description> 
    <language>en-US</language> 
    <tags>example</tags> 
  </metadata> 
</package> 
```

Then run the following to create the package: 

```bash
$ dotnet pack --configuration Release 
```

## Step 3: Adding the Feed to NuGet CLI { #add-feed }

To add your `internal-nuget` feed to NuGet CLI, you will need the feed URL. This is found on the top right of the feed page:

![](/resources/docs/proget-nuget-internal-url.png){height="" width="50%"}

Then add the feed as a source to your NuGet client by entering:

```bash
$ dotnet nuget add source https://«proget-server»/nuget/public-nuget/v3/index.json --name internal-nuget
```

## Step 4: Setting Up Authentication to Your NuGet Feed.

To push packages to a feed you will need to configure authentication using a "NuGet API Key". To authenticate you can either create an API Key in ProGet or use `«username:password»` as one. To learn how to do either, see [Authenticating to a NuGet Feed](/docs/proget/feeds/nuget#authenticating-to-nuget-feeds).

## Step 5: Pushing Your Package to Your NuGet Feed

To push your NuGet package to your NuGet feed, first make sure you have [added the feed as a source](#add-feed) with [authentication](/docs/proget/feeds/nuget#authenticating-to-nuget-feeds), and then use the `dotnet nuget push` command:

```bash
$ dotnet nuget push «package-name» -Source «source-name»
```

Or if you are using `«username:password»` to authenticate:

```bash
$ dotnet nuget push «package-name» -Source «source-name» -ApiKey «username:password»
```

### Examples:

Pushing the package `MyPackage` to a configured source named `internal-nuget`, you would enter:

```bash
$ dotnet nuget push bin/Release/MyPackage.1.0.0.nupkg -Source internal-nuget
```

Or using the username `johnsmith` and password `abc12345` you would enter:

```bash
$ dotnet nuget push bin/Release/MyPackage.1.0.0.nupkg -Source internal-nuget -ApiKey johnsmith:abc12345
```

Your package will then be uploaded and appear in your NuGet feed.

![](/resources/docs/proget-nuget-internal-uploaded.png){height="" width="50%"}